# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    Optional as typing___Optional,
    Text as typing___Text,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


class Resource(google___protobuf___message___Message):
    max_capacity = ... # type: int
    min_capacity = ... # type: int
    renewable = ... # type: bool
    unit_cost = ... # type: int

    def __init__(self,
        *,
        max_capacity : typing___Optional[int] = None,
        min_capacity : typing___Optional[int] = None,
        renewable : typing___Optional[bool] = None,
        unit_cost : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> Resource: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"max_capacity",u"min_capacity",u"renewable",u"unit_cost"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"max_capacity",b"max_capacity",u"min_capacity",b"min_capacity",u"renewable",b"renewable",u"unit_cost",b"unit_cost"]) -> None: ...

class Recipe(google___protobuf___message___Message):
    duration = ... # type: int
    demands = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    resources = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        duration : typing___Optional[int] = None,
        demands : typing___Optional[typing___Iterable[int]] = None,
        resources : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> Recipe: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"demands",u"duration",u"resources"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"demands",b"demands",u"duration",b"duration",u"resources",b"resources"]) -> None: ...

class PerRecipeDelays(google___protobuf___message___Message):
    min_delays = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        min_delays : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> PerRecipeDelays: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"min_delays"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"min_delays",b"min_delays"]) -> None: ...

class PerSuccessorDelays(google___protobuf___message___Message):

    @property
    def recipe_delays(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[PerRecipeDelays]: ...

    def __init__(self,
        *,
        recipe_delays : typing___Optional[typing___Iterable[PerRecipeDelays]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> PerSuccessorDelays: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"recipe_delays"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"recipe_delays",b"recipe_delays"]) -> None: ...

class Task(google___protobuf___message___Message):
    successors = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    @property
    def recipes(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Recipe]: ...

    @property
    def successor_delays(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[PerSuccessorDelays]: ...

    def __init__(self,
        *,
        successors : typing___Optional[typing___Iterable[int]] = None,
        recipes : typing___Optional[typing___Iterable[Recipe]] = None,
        successor_delays : typing___Optional[typing___Iterable[PerSuccessorDelays]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> Task: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"recipes",u"successor_delays",u"successors"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"recipes",b"recipes",u"successor_delays",b"successor_delays",u"successors",b"successors"]) -> None: ...

class RcpspProblem(google___protobuf___message___Message):
    is_consumer_producer = ... # type: bool
    is_resource_investment = ... # type: bool
    is_rcpsp_max = ... # type: bool
    deadline = ... # type: int
    horizon = ... # type: int
    release_date = ... # type: int
    tardiness_cost = ... # type: int
    mpm_time = ... # type: int
    seed = ... # type: int
    basedata = ... # type: typing___Text
    due_date = ... # type: int
    name = ... # type: typing___Text

    @property
    def resources(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Resource]: ...

    @property
    def tasks(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Task]: ...

    def __init__(self,
        *,
        resources : typing___Optional[typing___Iterable[Resource]] = None,
        tasks : typing___Optional[typing___Iterable[Task]] = None,
        is_consumer_producer : typing___Optional[bool] = None,
        is_resource_investment : typing___Optional[bool] = None,
        is_rcpsp_max : typing___Optional[bool] = None,
        deadline : typing___Optional[int] = None,
        horizon : typing___Optional[int] = None,
        release_date : typing___Optional[int] = None,
        tardiness_cost : typing___Optional[int] = None,
        mpm_time : typing___Optional[int] = None,
        seed : typing___Optional[int] = None,
        basedata : typing___Optional[typing___Text] = None,
        due_date : typing___Optional[int] = None,
        name : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> RcpspProblem: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"basedata",u"deadline",u"due_date",u"horizon",u"is_consumer_producer",u"is_rcpsp_max",u"is_resource_investment",u"mpm_time",u"name",u"release_date",u"resources",u"seed",u"tardiness_cost",u"tasks"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"basedata",b"basedata",u"deadline",b"deadline",u"due_date",b"due_date",u"horizon",b"horizon",u"is_consumer_producer",b"is_consumer_producer",u"is_rcpsp_max",b"is_rcpsp_max",u"is_resource_investment",b"is_resource_investment",u"mpm_time",b"mpm_time",u"name",b"name",u"release_date",b"release_date",u"resources",b"resources",u"seed",b"seed",u"tardiness_cost",b"tardiness_cost",u"tasks",b"tasks"]) -> None: ...



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    EnumDescriptor as google___protobuf___descriptor___EnumDescriptor,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    List as typing___List,
    Tuple as typing___Tuple,
    cast as typing___cast,
)


class FirstSolutionStrategy(google___protobuf___message___Message):
    class Value(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> FirstSolutionStrategy.Value: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[FirstSolutionStrategy.Value]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, FirstSolutionStrategy.Value]]: ...
        UNSET = typing___cast(FirstSolutionStrategy.Value, 0)
        AUTOMATIC = typing___cast(FirstSolutionStrategy.Value, 15)
        PATH_CHEAPEST_ARC = typing___cast(FirstSolutionStrategy.Value, 3)
        PATH_MOST_CONSTRAINED_ARC = typing___cast(FirstSolutionStrategy.Value, 4)
        EVALUATOR_STRATEGY = typing___cast(FirstSolutionStrategy.Value, 5)
        SAVINGS = typing___cast(FirstSolutionStrategy.Value, 10)
        SWEEP = typing___cast(FirstSolutionStrategy.Value, 11)
        CHRISTOFIDES = typing___cast(FirstSolutionStrategy.Value, 13)
        ALL_UNPERFORMED = typing___cast(FirstSolutionStrategy.Value, 6)
        BEST_INSERTION = typing___cast(FirstSolutionStrategy.Value, 7)
        PARALLEL_CHEAPEST_INSERTION = typing___cast(FirstSolutionStrategy.Value, 8)
        SEQUENTIAL_CHEAPEST_INSERTION = typing___cast(FirstSolutionStrategy.Value, 14)
        LOCAL_CHEAPEST_INSERTION = typing___cast(FirstSolutionStrategy.Value, 9)
        GLOBAL_CHEAPEST_ARC = typing___cast(FirstSolutionStrategy.Value, 1)
        LOCAL_CHEAPEST_ARC = typing___cast(FirstSolutionStrategy.Value, 2)
        FIRST_UNBOUND_MIN_VALUE = typing___cast(FirstSolutionStrategy.Value, 12)
    UNSET = typing___cast(FirstSolutionStrategy.Value, 0)
    AUTOMATIC = typing___cast(FirstSolutionStrategy.Value, 15)
    PATH_CHEAPEST_ARC = typing___cast(FirstSolutionStrategy.Value, 3)
    PATH_MOST_CONSTRAINED_ARC = typing___cast(FirstSolutionStrategy.Value, 4)
    EVALUATOR_STRATEGY = typing___cast(FirstSolutionStrategy.Value, 5)
    SAVINGS = typing___cast(FirstSolutionStrategy.Value, 10)
    SWEEP = typing___cast(FirstSolutionStrategy.Value, 11)
    CHRISTOFIDES = typing___cast(FirstSolutionStrategy.Value, 13)
    ALL_UNPERFORMED = typing___cast(FirstSolutionStrategy.Value, 6)
    BEST_INSERTION = typing___cast(FirstSolutionStrategy.Value, 7)
    PARALLEL_CHEAPEST_INSERTION = typing___cast(FirstSolutionStrategy.Value, 8)
    SEQUENTIAL_CHEAPEST_INSERTION = typing___cast(FirstSolutionStrategy.Value, 14)
    LOCAL_CHEAPEST_INSERTION = typing___cast(FirstSolutionStrategy.Value, 9)
    GLOBAL_CHEAPEST_ARC = typing___cast(FirstSolutionStrategy.Value, 1)
    LOCAL_CHEAPEST_ARC = typing___cast(FirstSolutionStrategy.Value, 2)
    FIRST_UNBOUND_MIN_VALUE = typing___cast(FirstSolutionStrategy.Value, 12)


    def __init__(self,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> FirstSolutionStrategy: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class LocalSearchMetaheuristic(google___protobuf___message___Message):
    class Value(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> LocalSearchMetaheuristic.Value: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[LocalSearchMetaheuristic.Value]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, LocalSearchMetaheuristic.Value]]: ...
        UNSET = typing___cast(LocalSearchMetaheuristic.Value, 0)
        AUTOMATIC = typing___cast(LocalSearchMetaheuristic.Value, 6)
        GREEDY_DESCENT = typing___cast(LocalSearchMetaheuristic.Value, 1)
        GUIDED_LOCAL_SEARCH = typing___cast(LocalSearchMetaheuristic.Value, 2)
        SIMULATED_ANNEALING = typing___cast(LocalSearchMetaheuristic.Value, 3)
        TABU_SEARCH = typing___cast(LocalSearchMetaheuristic.Value, 4)
        GENERIC_TABU_SEARCH = typing___cast(LocalSearchMetaheuristic.Value, 5)
    UNSET = typing___cast(LocalSearchMetaheuristic.Value, 0)
    AUTOMATIC = typing___cast(LocalSearchMetaheuristic.Value, 6)
    GREEDY_DESCENT = typing___cast(LocalSearchMetaheuristic.Value, 1)
    GUIDED_LOCAL_SEARCH = typing___cast(LocalSearchMetaheuristic.Value, 2)
    SIMULATED_ANNEALING = typing___cast(LocalSearchMetaheuristic.Value, 3)
    TABU_SEARCH = typing___cast(LocalSearchMetaheuristic.Value, 4)
    GENERIC_TABU_SEARCH = typing___cast(LocalSearchMetaheuristic.Value, 5)


    def __init__(self,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> LocalSearchMetaheuristic: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    EnumDescriptor as google___protobuf___descriptor___EnumDescriptor,
)

from google.protobuf.duration_pb2 import (
    Duration as google___protobuf___duration_pb2___Duration,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from ortools.constraint_solver.routing_enums_pb2 import (
    FirstSolutionStrategy as ortools___constraint_solver___routing_enums_pb2___FirstSolutionStrategy,
    LocalSearchMetaheuristic as ortools___constraint_solver___routing_enums_pb2___LocalSearchMetaheuristic,
)

from ortools.constraint_solver.solver_parameters_pb2 import (
    ConstraintSolverParameters as ortools___constraint_solver___solver_parameters_pb2___ConstraintSolverParameters,
)

from ortools.util.optional_boolean_pb2 import (
    OptionalBoolean as ortools___util___optional_boolean_pb2___OptionalBoolean,
)

from typing import (
    List as typing___List,
    Optional as typing___Optional,
    Tuple as typing___Tuple,
    cast as typing___cast,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


class RoutingSearchParameters(google___protobuf___message___Message):
    class SchedulingSolver(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> RoutingSearchParameters.SchedulingSolver: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[RoutingSearchParameters.SchedulingSolver]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, RoutingSearchParameters.SchedulingSolver]]: ...
        UNSET = typing___cast(RoutingSearchParameters.SchedulingSolver, 0)
        GLOP = typing___cast(RoutingSearchParameters.SchedulingSolver, 1)
        CP_SAT = typing___cast(RoutingSearchParameters.SchedulingSolver, 2)
    UNSET = typing___cast(RoutingSearchParameters.SchedulingSolver, 0)
    GLOP = typing___cast(RoutingSearchParameters.SchedulingSolver, 1)
    CP_SAT = typing___cast(RoutingSearchParameters.SchedulingSolver, 2)

    class LocalSearchNeighborhoodOperators(google___protobuf___message___Message):
        use_relocate = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_relocate_pair = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_light_relocate_pair = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_relocate_neighbors = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_relocate_subtrip = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_exchange = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_exchange_pair = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_exchange_subtrip = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_cross = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_cross_exchange = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_relocate_expensive_chain = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_two_opt = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_or_opt = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_lin_kernighan = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_tsp_opt = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_make_active = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_relocate_and_make_active = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_make_inactive = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_make_chain_inactive = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_swap_active = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_extended_swap_active = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_node_pair_swap_active = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_path_lns = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_full_path_lns = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_tsp_lns = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_inactive_lns = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_global_cheapest_insertion_path_lns = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_local_cheapest_insertion_path_lns = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_global_cheapest_insertion_expensive_chain_lns = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
        use_local_cheapest_insertion_expensive_chain_lns = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean

        def __init__(self,
            *,
            use_relocate : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_relocate_pair : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_light_relocate_pair : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_relocate_neighbors : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_relocate_subtrip : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_exchange : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_exchange_pair : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_exchange_subtrip : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_cross : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_cross_exchange : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_relocate_expensive_chain : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_two_opt : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_or_opt : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_lin_kernighan : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_tsp_opt : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_make_active : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_relocate_and_make_active : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_make_inactive : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_make_chain_inactive : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_swap_active : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_extended_swap_active : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_node_pair_swap_active : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_path_lns : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_full_path_lns : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_tsp_lns : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_inactive_lns : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_global_cheapest_insertion_path_lns : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_local_cheapest_insertion_path_lns : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_global_cheapest_insertion_expensive_chain_lns : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            use_local_cheapest_insertion_expensive_chain_lns : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
            ) -> None: ...
        @classmethod
        def FromString(cls, s: bytes) -> RoutingSearchParameters.LocalSearchNeighborhoodOperators: ...
        def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        if sys.version_info >= (3,):
            def ClearField(self, field_name: typing_extensions___Literal[u"use_cross",u"use_cross_exchange",u"use_exchange",u"use_exchange_pair",u"use_exchange_subtrip",u"use_extended_swap_active",u"use_full_path_lns",u"use_global_cheapest_insertion_expensive_chain_lns",u"use_global_cheapest_insertion_path_lns",u"use_inactive_lns",u"use_light_relocate_pair",u"use_lin_kernighan",u"use_local_cheapest_insertion_expensive_chain_lns",u"use_local_cheapest_insertion_path_lns",u"use_make_active",u"use_make_chain_inactive",u"use_make_inactive",u"use_node_pair_swap_active",u"use_or_opt",u"use_path_lns",u"use_relocate",u"use_relocate_and_make_active",u"use_relocate_expensive_chain",u"use_relocate_neighbors",u"use_relocate_pair",u"use_relocate_subtrip",u"use_swap_active",u"use_tsp_lns",u"use_tsp_opt",u"use_two_opt"]) -> None: ...
        else:
            def ClearField(self, field_name: typing_extensions___Literal[u"use_cross",b"use_cross",u"use_cross_exchange",b"use_cross_exchange",u"use_exchange",b"use_exchange",u"use_exchange_pair",b"use_exchange_pair",u"use_exchange_subtrip",b"use_exchange_subtrip",u"use_extended_swap_active",b"use_extended_swap_active",u"use_full_path_lns",b"use_full_path_lns",u"use_global_cheapest_insertion_expensive_chain_lns",b"use_global_cheapest_insertion_expensive_chain_lns",u"use_global_cheapest_insertion_path_lns",b"use_global_cheapest_insertion_path_lns",u"use_inactive_lns",b"use_inactive_lns",u"use_light_relocate_pair",b"use_light_relocate_pair",u"use_lin_kernighan",b"use_lin_kernighan",u"use_local_cheapest_insertion_expensive_chain_lns",b"use_local_cheapest_insertion_expensive_chain_lns",u"use_local_cheapest_insertion_path_lns",b"use_local_cheapest_insertion_path_lns",u"use_make_active",b"use_make_active",u"use_make_chain_inactive",b"use_make_chain_inactive",u"use_make_inactive",b"use_make_inactive",u"use_node_pair_swap_active",b"use_node_pair_swap_active",u"use_or_opt",b"use_or_opt",u"use_path_lns",b"use_path_lns",u"use_relocate",b"use_relocate",u"use_relocate_and_make_active",b"use_relocate_and_make_active",u"use_relocate_expensive_chain",b"use_relocate_expensive_chain",u"use_relocate_neighbors",b"use_relocate_neighbors",u"use_relocate_pair",b"use_relocate_pair",u"use_relocate_subtrip",b"use_relocate_subtrip",u"use_swap_active",b"use_swap_active",u"use_tsp_lns",b"use_tsp_lns",u"use_tsp_opt",b"use_tsp_opt",u"use_two_opt",b"use_two_opt"]) -> None: ...

    first_solution_strategy = ... # type: ortools___constraint_solver___routing_enums_pb2___FirstSolutionStrategy.Value
    use_unfiltered_first_solution_strategy = ... # type: bool
    savings_neighbors_ratio = ... # type: float
    savings_max_memory_usage_bytes = ... # type: float
    savings_add_reverse_arcs = ... # type: bool
    savings_arc_coefficient = ... # type: float
    savings_parallel_routes = ... # type: bool
    cheapest_insertion_farthest_seeds_ratio = ... # type: float
    cheapest_insertion_first_solution_neighbors_ratio = ... # type: float
    cheapest_insertion_ls_operator_neighbors_ratio = ... # type: float
    christofides_use_minimum_matching = ... # type: bool
    relocate_expensive_chain_num_arcs_to_consider = ... # type: int
    heuristic_expensive_chain_lns_num_arcs_to_consider = ... # type: int
    local_search_metaheuristic = ... # type: ortools___constraint_solver___routing_enums_pb2___LocalSearchMetaheuristic.Value
    guided_local_search_lambda_coefficient = ... # type: float
    use_depth_first_search = ... # type: bool
    use_cp = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
    use_cp_sat = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
    continuous_scheduling_solver = ... # type: RoutingSearchParameters.SchedulingSolver
    mixed_integer_scheduling_solver = ... # type: RoutingSearchParameters.SchedulingSolver
    optimization_step = ... # type: float
    number_of_solutions_to_collect = ... # type: int
    solution_limit = ... # type: int
    use_full_propagation = ... # type: bool
    log_search = ... # type: bool
    log_cost_scaling_factor = ... # type: float
    log_cost_offset = ... # type: float

    @property
    def local_search_operators(self) -> RoutingSearchParameters.LocalSearchNeighborhoodOperators: ...

    @property
    def time_limit(self) -> google___protobuf___duration_pb2___Duration: ...

    @property
    def lns_time_limit(self) -> google___protobuf___duration_pb2___Duration: ...

    def __init__(self,
        *,
        first_solution_strategy : typing___Optional[ortools___constraint_solver___routing_enums_pb2___FirstSolutionStrategy.Value] = None,
        use_unfiltered_first_solution_strategy : typing___Optional[bool] = None,
        savings_neighbors_ratio : typing___Optional[float] = None,
        savings_max_memory_usage_bytes : typing___Optional[float] = None,
        savings_add_reverse_arcs : typing___Optional[bool] = None,
        savings_arc_coefficient : typing___Optional[float] = None,
        savings_parallel_routes : typing___Optional[bool] = None,
        cheapest_insertion_farthest_seeds_ratio : typing___Optional[float] = None,
        cheapest_insertion_first_solution_neighbors_ratio : typing___Optional[float] = None,
        cheapest_insertion_ls_operator_neighbors_ratio : typing___Optional[float] = None,
        christofides_use_minimum_matching : typing___Optional[bool] = None,
        local_search_operators : typing___Optional[RoutingSearchParameters.LocalSearchNeighborhoodOperators] = None,
        relocate_expensive_chain_num_arcs_to_consider : typing___Optional[int] = None,
        heuristic_expensive_chain_lns_num_arcs_to_consider : typing___Optional[int] = None,
        local_search_metaheuristic : typing___Optional[ortools___constraint_solver___routing_enums_pb2___LocalSearchMetaheuristic.Value] = None,
        guided_local_search_lambda_coefficient : typing___Optional[float] = None,
        use_depth_first_search : typing___Optional[bool] = None,
        use_cp : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
        use_cp_sat : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
        continuous_scheduling_solver : typing___Optional[RoutingSearchParameters.SchedulingSolver] = None,
        mixed_integer_scheduling_solver : typing___Optional[RoutingSearchParameters.SchedulingSolver] = None,
        optimization_step : typing___Optional[float] = None,
        number_of_solutions_to_collect : typing___Optional[int] = None,
        solution_limit : typing___Optional[int] = None,
        time_limit : typing___Optional[google___protobuf___duration_pb2___Duration] = None,
        lns_time_limit : typing___Optional[google___protobuf___duration_pb2___Duration] = None,
        use_full_propagation : typing___Optional[bool] = None,
        log_search : typing___Optional[bool] = None,
        log_cost_scaling_factor : typing___Optional[float] = None,
        log_cost_offset : typing___Optional[float] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> RoutingSearchParameters: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"lns_time_limit",u"local_search_operators",u"time_limit"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"cheapest_insertion_farthest_seeds_ratio",u"cheapest_insertion_first_solution_neighbors_ratio",u"cheapest_insertion_ls_operator_neighbors_ratio",u"christofides_use_minimum_matching",u"continuous_scheduling_solver",u"first_solution_strategy",u"guided_local_search_lambda_coefficient",u"heuristic_expensive_chain_lns_num_arcs_to_consider",u"lns_time_limit",u"local_search_metaheuristic",u"local_search_operators",u"log_cost_offset",u"log_cost_scaling_factor",u"log_search",u"mixed_integer_scheduling_solver",u"number_of_solutions_to_collect",u"optimization_step",u"relocate_expensive_chain_num_arcs_to_consider",u"savings_add_reverse_arcs",u"savings_arc_coefficient",u"savings_max_memory_usage_bytes",u"savings_neighbors_ratio",u"savings_parallel_routes",u"solution_limit",u"time_limit",u"use_cp",u"use_cp_sat",u"use_depth_first_search",u"use_full_propagation",u"use_unfiltered_first_solution_strategy"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"lns_time_limit",b"lns_time_limit",u"local_search_operators",b"local_search_operators",u"time_limit",b"time_limit"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"cheapest_insertion_farthest_seeds_ratio",b"cheapest_insertion_farthest_seeds_ratio",u"cheapest_insertion_first_solution_neighbors_ratio",b"cheapest_insertion_first_solution_neighbors_ratio",u"cheapest_insertion_ls_operator_neighbors_ratio",b"cheapest_insertion_ls_operator_neighbors_ratio",u"christofides_use_minimum_matching",b"christofides_use_minimum_matching",u"continuous_scheduling_solver",b"continuous_scheduling_solver",u"first_solution_strategy",b"first_solution_strategy",u"guided_local_search_lambda_coefficient",b"guided_local_search_lambda_coefficient",u"heuristic_expensive_chain_lns_num_arcs_to_consider",b"heuristic_expensive_chain_lns_num_arcs_to_consider",u"lns_time_limit",b"lns_time_limit",u"local_search_metaheuristic",b"local_search_metaheuristic",u"local_search_operators",b"local_search_operators",u"log_cost_offset",b"log_cost_offset",u"log_cost_scaling_factor",b"log_cost_scaling_factor",u"log_search",b"log_search",u"mixed_integer_scheduling_solver",b"mixed_integer_scheduling_solver",u"number_of_solutions_to_collect",b"number_of_solutions_to_collect",u"optimization_step",b"optimization_step",u"relocate_expensive_chain_num_arcs_to_consider",b"relocate_expensive_chain_num_arcs_to_consider",u"savings_add_reverse_arcs",b"savings_add_reverse_arcs",u"savings_arc_coefficient",b"savings_arc_coefficient",u"savings_max_memory_usage_bytes",b"savings_max_memory_usage_bytes",u"savings_neighbors_ratio",b"savings_neighbors_ratio",u"savings_parallel_routes",b"savings_parallel_routes",u"solution_limit",b"solution_limit",u"time_limit",b"time_limit",u"use_cp",b"use_cp",u"use_cp_sat",b"use_cp_sat",u"use_depth_first_search",b"use_depth_first_search",u"use_full_propagation",b"use_full_propagation",u"use_unfiltered_first_solution_strategy",b"use_unfiltered_first_solution_strategy"]) -> None: ...

class RoutingModelParameters(google___protobuf___message___Message):
    reduce_vehicle_cost_model = ... # type: bool
    max_callback_cache_size = ... # type: int

    @property
    def solver_parameters(self) -> ortools___constraint_solver___solver_parameters_pb2___ConstraintSolverParameters: ...

    def __init__(self,
        *,
        solver_parameters : typing___Optional[ortools___constraint_solver___solver_parameters_pb2___ConstraintSolverParameters] = None,
        reduce_vehicle_cost_model : typing___Optional[bool] = None,
        max_callback_cache_size : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> RoutingModelParameters: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"solver_parameters"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"max_callback_cache_size",u"reduce_vehicle_cost_model",u"solver_parameters"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"solver_parameters",b"solver_parameters"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"max_callback_cache_size",b"max_callback_cache_size",u"reduce_vehicle_cost_model",b"reduce_vehicle_cost_model",u"solver_parameters",b"solver_parameters"]) -> None: ...



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    EnumDescriptor as google___protobuf___descriptor___EnumDescriptor,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    List as typing___List,
    Optional as typing___Optional,
    Text as typing___Text,
    Tuple as typing___Tuple,
    cast as typing___cast,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


class ConstraintSolverParameters(google___protobuf___message___Message):
    class TrailCompression(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> ConstraintSolverParameters.TrailCompression: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[ConstraintSolverParameters.TrailCompression]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, ConstraintSolverParameters.TrailCompression]]: ...
        NO_COMPRESSION = typing___cast(ConstraintSolverParameters.TrailCompression, 0)
        COMPRESS_WITH_ZLIB = typing___cast(ConstraintSolverParameters.TrailCompression, 1)
    NO_COMPRESSION = typing___cast(ConstraintSolverParameters.TrailCompression, 0)
    COMPRESS_WITH_ZLIB = typing___cast(ConstraintSolverParameters.TrailCompression, 1)

    compress_trail = ... # type: ConstraintSolverParameters.TrailCompression
    trail_block_size = ... # type: int
    array_split_size = ... # type: int
    store_names = ... # type: bool
    name_cast_variables = ... # type: bool
    name_all_variables = ... # type: bool
    profile_propagation = ... # type: bool
    profile_file = ... # type: typing___Text
    profile_local_search = ... # type: bool
    print_local_search_profile = ... # type: bool
    trace_propagation = ... # type: bool
    trace_search = ... # type: bool
    print_model = ... # type: bool
    print_model_stats = ... # type: bool
    print_added_constraints = ... # type: bool
    disable_solve = ... # type: bool
    use_small_table = ... # type: bool
    use_cumulative_edge_finder = ... # type: bool
    use_cumulative_time_table = ... # type: bool
    use_cumulative_time_table_sync = ... # type: bool
    use_sequence_high_demand_tasks = ... # type: bool
    use_all_possible_disjunctions = ... # type: bool
    max_edge_finder_size = ... # type: int
    diffn_use_cumulative = ... # type: bool
    use_element_rmq = ... # type: bool
    skip_locally_optimal_paths = ... # type: bool
    check_solution_period = ... # type: int

    def __init__(self,
        *,
        compress_trail : typing___Optional[ConstraintSolverParameters.TrailCompression] = None,
        trail_block_size : typing___Optional[int] = None,
        array_split_size : typing___Optional[int] = None,
        store_names : typing___Optional[bool] = None,
        name_cast_variables : typing___Optional[bool] = None,
        name_all_variables : typing___Optional[bool] = None,
        profile_propagation : typing___Optional[bool] = None,
        profile_file : typing___Optional[typing___Text] = None,
        profile_local_search : typing___Optional[bool] = None,
        print_local_search_profile : typing___Optional[bool] = None,
        trace_propagation : typing___Optional[bool] = None,
        trace_search : typing___Optional[bool] = None,
        print_model : typing___Optional[bool] = None,
        print_model_stats : typing___Optional[bool] = None,
        print_added_constraints : typing___Optional[bool] = None,
        disable_solve : typing___Optional[bool] = None,
        use_small_table : typing___Optional[bool] = None,
        use_cumulative_edge_finder : typing___Optional[bool] = None,
        use_cumulative_time_table : typing___Optional[bool] = None,
        use_cumulative_time_table_sync : typing___Optional[bool] = None,
        use_sequence_high_demand_tasks : typing___Optional[bool] = None,
        use_all_possible_disjunctions : typing___Optional[bool] = None,
        max_edge_finder_size : typing___Optional[int] = None,
        diffn_use_cumulative : typing___Optional[bool] = None,
        use_element_rmq : typing___Optional[bool] = None,
        skip_locally_optimal_paths : typing___Optional[bool] = None,
        check_solution_period : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> ConstraintSolverParameters: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"array_split_size",u"check_solution_period",u"compress_trail",u"diffn_use_cumulative",u"disable_solve",u"max_edge_finder_size",u"name_all_variables",u"name_cast_variables",u"print_added_constraints",u"print_local_search_profile",u"print_model",u"print_model_stats",u"profile_file",u"profile_local_search",u"profile_propagation",u"skip_locally_optimal_paths",u"store_names",u"trace_propagation",u"trace_search",u"trail_block_size",u"use_all_possible_disjunctions",u"use_cumulative_edge_finder",u"use_cumulative_time_table",u"use_cumulative_time_table_sync",u"use_element_rmq",u"use_sequence_high_demand_tasks",u"use_small_table"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"array_split_size",b"array_split_size",u"check_solution_period",b"check_solution_period",u"compress_trail",b"compress_trail",u"diffn_use_cumulative",b"diffn_use_cumulative",u"disable_solve",b"disable_solve",u"max_edge_finder_size",b"max_edge_finder_size",u"name_all_variables",b"name_all_variables",u"name_cast_variables",b"name_cast_variables",u"print_added_constraints",b"print_added_constraints",u"print_local_search_profile",b"print_local_search_profile",u"print_model",b"print_model",u"print_model_stats",b"print_model_stats",u"profile_file",b"profile_file",u"profile_local_search",b"profile_local_search",u"profile_propagation",b"profile_propagation",u"skip_locally_optimal_paths",b"skip_locally_optimal_paths",u"store_names",b"store_names",u"trace_propagation",b"trace_propagation",u"trace_search",b"trace_search",u"trail_block_size",b"trail_block_size",u"use_all_possible_disjunctions",b"use_all_possible_disjunctions",u"use_cumulative_edge_finder",b"use_cumulative_edge_finder",u"use_cumulative_time_table",b"use_cumulative_time_table",u"use_cumulative_time_table_sync",b"use_cumulative_time_table_sync",u"use_element_rmq",b"use_element_rmq",u"use_sequence_high_demand_tasks",b"use_sequence_high_demand_tasks",u"use_small_table",b"use_small_table"]) -> None: ...



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    EnumDescriptor as google___protobuf___descriptor___EnumDescriptor,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    List as typing___List,
    Tuple as typing___Tuple,
    cast as typing___cast,
)


class OptionalBoolean(int):
    DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
    @classmethod
    def Name(cls, number: int) -> str: ...
    @classmethod
    def Value(cls, name: str) -> OptionalBoolean: ...
    @classmethod
    def keys(cls) -> typing___List[str]: ...
    @classmethod
    def values(cls) -> typing___List[OptionalBoolean]: ...
    @classmethod
    def items(cls) -> typing___List[typing___Tuple[str, OptionalBoolean]]: ...
    BOOL_UNSPECIFIED = typing___cast(OptionalBoolean, 0)
    BOOL_FALSE = typing___cast(OptionalBoolean, 2)
    BOOL_TRUE = typing___cast(OptionalBoolean, 3)
BOOL_UNSPECIFIED = typing___cast(OptionalBoolean, 0)
BOOL_FALSE = typing___cast(OptionalBoolean, 2)
BOOL_TRUE = typing___cast(OptionalBoolean, 3)



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Optional as typing___Optional,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


class RegularLimitParameters(google___protobuf___message___Message):
    time = ... # type: int
    branches = ... # type: int
    failures = ... # type: int
    solutions = ... # type: int
    smart_time_check = ... # type: bool
    cumulative = ... # type: bool

    def __init__(self,
        *,
        time : typing___Optional[int] = None,
        branches : typing___Optional[int] = None,
        failures : typing___Optional[int] = None,
        solutions : typing___Optional[int] = None,
        smart_time_check : typing___Optional[bool] = None,
        cumulative : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> RegularLimitParameters: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"branches",u"cumulative",u"failures",u"smart_time_check",u"solutions",u"time"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"branches",b"branches",u"cumulative",b"cumulative",u"failures",b"failures",u"smart_time_check",b"smart_time_check",u"solutions",b"solutions",u"time",b"time"]) -> None: ...



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    Optional as typing___Optional,
    Text as typing___Text,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


class IntVarAssignment(google___protobuf___message___Message):
    var_id = ... # type: typing___Text
    min = ... # type: int
    max = ... # type: int
    active = ... # type: bool

    def __init__(self,
        *,
        var_id : typing___Optional[typing___Text] = None,
        min : typing___Optional[int] = None,
        max : typing___Optional[int] = None,
        active : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> IntVarAssignment: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"active",u"max",u"min",u"var_id"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"active",b"active",u"max",b"max",u"min",b"min",u"var_id",b"var_id"]) -> None: ...

class IntervalVarAssignment(google___protobuf___message___Message):
    var_id = ... # type: typing___Text
    start_min = ... # type: int
    start_max = ... # type: int
    duration_min = ... # type: int
    duration_max = ... # type: int
    end_min = ... # type: int
    end_max = ... # type: int
    performed_min = ... # type: int
    performed_max = ... # type: int
    active = ... # type: bool

    def __init__(self,
        *,
        var_id : typing___Optional[typing___Text] = None,
        start_min : typing___Optional[int] = None,
        start_max : typing___Optional[int] = None,
        duration_min : typing___Optional[int] = None,
        duration_max : typing___Optional[int] = None,
        end_min : typing___Optional[int] = None,
        end_max : typing___Optional[int] = None,
        performed_min : typing___Optional[int] = None,
        performed_max : typing___Optional[int] = None,
        active : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> IntervalVarAssignment: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"active",u"duration_max",u"duration_min",u"end_max",u"end_min",u"performed_max",u"performed_min",u"start_max",u"start_min",u"var_id"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"active",b"active",u"duration_max",b"duration_max",u"duration_min",b"duration_min",u"end_max",b"end_max",u"end_min",b"end_min",u"performed_max",b"performed_max",u"performed_min",b"performed_min",u"start_max",b"start_max",u"start_min",b"start_min",u"var_id",b"var_id"]) -> None: ...

class SequenceVarAssignment(google___protobuf___message___Message):
    var_id = ... # type: typing___Text
    forward_sequence = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    backward_sequence = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    unperformed = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    active = ... # type: bool

    def __init__(self,
        *,
        var_id : typing___Optional[typing___Text] = None,
        forward_sequence : typing___Optional[typing___Iterable[int]] = None,
        backward_sequence : typing___Optional[typing___Iterable[int]] = None,
        unperformed : typing___Optional[typing___Iterable[int]] = None,
        active : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> SequenceVarAssignment: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"active",u"backward_sequence",u"forward_sequence",u"unperformed",u"var_id"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"active",b"active",u"backward_sequence",b"backward_sequence",u"forward_sequence",b"forward_sequence",u"unperformed",b"unperformed",u"var_id",b"var_id"]) -> None: ...

class WorkerInfo(google___protobuf___message___Message):
    worker_id = ... # type: int
    bns = ... # type: typing___Text

    def __init__(self,
        *,
        worker_id : typing___Optional[int] = None,
        bns : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> WorkerInfo: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"bns",u"worker_id"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"bns",b"bns",u"worker_id",b"worker_id"]) -> None: ...

class AssignmentProto(google___protobuf___message___Message):
    is_valid = ... # type: bool

    @property
    def int_var_assignment(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[IntVarAssignment]: ...

    @property
    def interval_var_assignment(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[IntervalVarAssignment]: ...

    @property
    def sequence_var_assignment(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[SequenceVarAssignment]: ...

    @property
    def objective(self) -> IntVarAssignment: ...

    @property
    def worker_info(self) -> WorkerInfo: ...

    def __init__(self,
        *,
        int_var_assignment : typing___Optional[typing___Iterable[IntVarAssignment]] = None,
        interval_var_assignment : typing___Optional[typing___Iterable[IntervalVarAssignment]] = None,
        sequence_var_assignment : typing___Optional[typing___Iterable[SequenceVarAssignment]] = None,
        objective : typing___Optional[IntVarAssignment] = None,
        worker_info : typing___Optional[WorkerInfo] = None,
        is_valid : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> AssignmentProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"objective",u"worker_info"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"int_var_assignment",u"interval_var_assignment",u"is_valid",u"objective",u"sequence_var_assignment",u"worker_info"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"objective",b"objective",u"worker_info",b"worker_info"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"int_var_assignment",b"int_var_assignment",u"interval_var_assignment",b"interval_var_assignment",u"is_valid",b"is_valid",u"objective",b"objective",u"sequence_var_assignment",b"sequence_var_assignment",u"worker_info",b"worker_info"]) -> None: ...



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    EnumDescriptor as google___protobuf___descriptor___EnumDescriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from ortools.util.optional_boolean_pb2 import (
    OptionalBoolean as ortools___util___optional_boolean_pb2___OptionalBoolean,
)

from typing import (
    Iterable as typing___Iterable,
    List as typing___List,
    Mapping as typing___Mapping,
    MutableMapping as typing___MutableMapping,
    Optional as typing___Optional,
    Text as typing___Text,
    Tuple as typing___Tuple,
    cast as typing___cast,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


class MPSolverResponseStatus(int):
    DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
    @classmethod
    def Name(cls, number: int) -> str: ...
    @classmethod
    def Value(cls, name: str) -> MPSolverResponseStatus: ...
    @classmethod
    def keys(cls) -> typing___List[str]: ...
    @classmethod
    def values(cls) -> typing___List[MPSolverResponseStatus]: ...
    @classmethod
    def items(cls) -> typing___List[typing___Tuple[str, MPSolverResponseStatus]]: ...
    MPSOLVER_OPTIMAL = typing___cast(MPSolverResponseStatus, 0)
    MPSOLVER_FEASIBLE = typing___cast(MPSolverResponseStatus, 1)
    MPSOLVER_INFEASIBLE = typing___cast(MPSolverResponseStatus, 2)
    MPSOLVER_UNBOUNDED = typing___cast(MPSolverResponseStatus, 3)
    MPSOLVER_ABNORMAL = typing___cast(MPSolverResponseStatus, 4)
    MPSOLVER_NOT_SOLVED = typing___cast(MPSolverResponseStatus, 6)
    MPSOLVER_MODEL_IS_VALID = typing___cast(MPSolverResponseStatus, 97)
    MPSOLVER_UNKNOWN_STATUS = typing___cast(MPSolverResponseStatus, 99)
    MPSOLVER_MODEL_INVALID = typing___cast(MPSolverResponseStatus, 5)
    MPSOLVER_MODEL_INVALID_SOLUTION_HINT = typing___cast(MPSolverResponseStatus, 84)
    MPSOLVER_MODEL_INVALID_SOLVER_PARAMETERS = typing___cast(MPSolverResponseStatus, 85)
    MPSOLVER_SOLVER_TYPE_UNAVAILABLE = typing___cast(MPSolverResponseStatus, 7)
MPSOLVER_OPTIMAL = typing___cast(MPSolverResponseStatus, 0)
MPSOLVER_FEASIBLE = typing___cast(MPSolverResponseStatus, 1)
MPSOLVER_INFEASIBLE = typing___cast(MPSolverResponseStatus, 2)
MPSOLVER_UNBOUNDED = typing___cast(MPSolverResponseStatus, 3)
MPSOLVER_ABNORMAL = typing___cast(MPSolverResponseStatus, 4)
MPSOLVER_NOT_SOLVED = typing___cast(MPSolverResponseStatus, 6)
MPSOLVER_MODEL_IS_VALID = typing___cast(MPSolverResponseStatus, 97)
MPSOLVER_UNKNOWN_STATUS = typing___cast(MPSolverResponseStatus, 99)
MPSOLVER_MODEL_INVALID = typing___cast(MPSolverResponseStatus, 5)
MPSOLVER_MODEL_INVALID_SOLUTION_HINT = typing___cast(MPSolverResponseStatus, 84)
MPSOLVER_MODEL_INVALID_SOLVER_PARAMETERS = typing___cast(MPSolverResponseStatus, 85)
MPSOLVER_SOLVER_TYPE_UNAVAILABLE = typing___cast(MPSolverResponseStatus, 7)

class MPVariableProto(google___protobuf___message___Message):
    lower_bound = ... # type: float
    upper_bound = ... # type: float
    objective_coefficient = ... # type: float
    is_integer = ... # type: bool
    name = ... # type: typing___Text
    branching_priority = ... # type: int

    def __init__(self,
        *,
        lower_bound : typing___Optional[float] = None,
        upper_bound : typing___Optional[float] = None,
        objective_coefficient : typing___Optional[float] = None,
        is_integer : typing___Optional[bool] = None,
        name : typing___Optional[typing___Text] = None,
        branching_priority : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPVariableProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"branching_priority",u"is_integer",u"lower_bound",u"name",u"objective_coefficient",u"upper_bound"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"branching_priority",u"is_integer",u"lower_bound",u"name",u"objective_coefficient",u"upper_bound"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"branching_priority",b"branching_priority",u"is_integer",b"is_integer",u"lower_bound",b"lower_bound",u"name",b"name",u"objective_coefficient",b"objective_coefficient",u"upper_bound",b"upper_bound"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"branching_priority",b"branching_priority",u"is_integer",b"is_integer",u"lower_bound",b"lower_bound",u"name",b"name",u"objective_coefficient",b"objective_coefficient",u"upper_bound",b"upper_bound"]) -> None: ...

class MPConstraintProto(google___protobuf___message___Message):
    var_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    coefficient = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[float]
    lower_bound = ... # type: float
    upper_bound = ... # type: float
    name = ... # type: typing___Text
    is_lazy = ... # type: bool

    def __init__(self,
        *,
        var_index : typing___Optional[typing___Iterable[int]] = None,
        coefficient : typing___Optional[typing___Iterable[float]] = None,
        lower_bound : typing___Optional[float] = None,
        upper_bound : typing___Optional[float] = None,
        name : typing___Optional[typing___Text] = None,
        is_lazy : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"is_lazy",u"lower_bound",u"name",u"upper_bound"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"coefficient",u"is_lazy",u"lower_bound",u"name",u"upper_bound",u"var_index"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"is_lazy",b"is_lazy",u"lower_bound",b"lower_bound",u"name",b"name",u"upper_bound",b"upper_bound"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"coefficient",b"coefficient",u"is_lazy",b"is_lazy",u"lower_bound",b"lower_bound",u"name",b"name",u"upper_bound",b"upper_bound",u"var_index",b"var_index"]) -> None: ...

class MPGeneralConstraintProto(google___protobuf___message___Message):
    name = ... # type: typing___Text

    @property
    def indicator_constraint(self) -> MPIndicatorConstraint: ...

    @property
    def sos_constraint(self) -> MPSosConstraint: ...

    @property
    def quadratic_constraint(self) -> MPQuadraticConstraint: ...

    @property
    def abs_constraint(self) -> MPAbsConstraint: ...

    @property
    def and_constraint(self) -> MPArrayConstraint: ...

    @property
    def or_constraint(self) -> MPArrayConstraint: ...

    @property
    def min_constraint(self) -> MPArrayWithConstantConstraint: ...

    @property
    def max_constraint(self) -> MPArrayWithConstantConstraint: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        indicator_constraint : typing___Optional[MPIndicatorConstraint] = None,
        sos_constraint : typing___Optional[MPSosConstraint] = None,
        quadratic_constraint : typing___Optional[MPQuadraticConstraint] = None,
        abs_constraint : typing___Optional[MPAbsConstraint] = None,
        and_constraint : typing___Optional[MPArrayConstraint] = None,
        or_constraint : typing___Optional[MPArrayConstraint] = None,
        min_constraint : typing___Optional[MPArrayWithConstantConstraint] = None,
        max_constraint : typing___Optional[MPArrayWithConstantConstraint] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPGeneralConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"abs_constraint",u"and_constraint",u"general_constraint",u"indicator_constraint",u"max_constraint",u"min_constraint",u"name",u"or_constraint",u"quadratic_constraint",u"sos_constraint"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"abs_constraint",u"and_constraint",u"general_constraint",u"indicator_constraint",u"max_constraint",u"min_constraint",u"name",u"or_constraint",u"quadratic_constraint",u"sos_constraint"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"abs_constraint",b"abs_constraint",u"and_constraint",b"and_constraint",u"general_constraint",b"general_constraint",u"indicator_constraint",b"indicator_constraint",u"max_constraint",b"max_constraint",u"min_constraint",b"min_constraint",u"name",b"name",u"or_constraint",b"or_constraint",u"quadratic_constraint",b"quadratic_constraint",u"sos_constraint",b"sos_constraint"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"abs_constraint",b"abs_constraint",u"and_constraint",b"and_constraint",u"general_constraint",b"general_constraint",u"indicator_constraint",b"indicator_constraint",u"max_constraint",b"max_constraint",u"min_constraint",b"min_constraint",u"name",b"name",u"or_constraint",b"or_constraint",u"quadratic_constraint",b"quadratic_constraint",u"sos_constraint",b"sos_constraint"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"general_constraint",b"general_constraint"]) -> typing_extensions___Literal["indicator_constraint","sos_constraint","quadratic_constraint","abs_constraint","and_constraint","or_constraint","min_constraint","max_constraint"]: ...

class MPIndicatorConstraint(google___protobuf___message___Message):
    var_index = ... # type: int
    var_value = ... # type: int

    @property
    def constraint(self) -> MPConstraintProto: ...

    def __init__(self,
        *,
        var_index : typing___Optional[int] = None,
        var_value : typing___Optional[int] = None,
        constraint : typing___Optional[MPConstraintProto] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPIndicatorConstraint: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"constraint",u"var_index",u"var_value"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"constraint",u"var_index",u"var_value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"constraint",b"constraint",u"var_index",b"var_index",u"var_value",b"var_value"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"constraint",b"constraint",u"var_index",b"var_index",u"var_value",b"var_value"]) -> None: ...

class MPSosConstraint(google___protobuf___message___Message):
    class Type(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> MPSosConstraint.Type: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[MPSosConstraint.Type]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, MPSosConstraint.Type]]: ...
        SOS1_DEFAULT = typing___cast(MPSosConstraint.Type, 0)
        SOS2 = typing___cast(MPSosConstraint.Type, 1)
    SOS1_DEFAULT = typing___cast(MPSosConstraint.Type, 0)
    SOS2 = typing___cast(MPSosConstraint.Type, 1)

    type = ... # type: MPSosConstraint.Type
    var_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    weight = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[float]

    def __init__(self,
        *,
        type : typing___Optional[MPSosConstraint.Type] = None,
        var_index : typing___Optional[typing___Iterable[int]] = None,
        weight : typing___Optional[typing___Iterable[float]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPSosConstraint: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"type"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"type",u"var_index",u"weight"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"type",b"type"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"type",b"type",u"var_index",b"var_index",u"weight",b"weight"]) -> None: ...

class MPQuadraticConstraint(google___protobuf___message___Message):
    var_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    coefficient = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[float]
    qvar1_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    qvar2_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    qcoefficient = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[float]
    lower_bound = ... # type: float
    upper_bound = ... # type: float

    def __init__(self,
        *,
        var_index : typing___Optional[typing___Iterable[int]] = None,
        coefficient : typing___Optional[typing___Iterable[float]] = None,
        qvar1_index : typing___Optional[typing___Iterable[int]] = None,
        qvar2_index : typing___Optional[typing___Iterable[int]] = None,
        qcoefficient : typing___Optional[typing___Iterable[float]] = None,
        lower_bound : typing___Optional[float] = None,
        upper_bound : typing___Optional[float] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPQuadraticConstraint: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"lower_bound",u"upper_bound"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"coefficient",u"lower_bound",u"qcoefficient",u"qvar1_index",u"qvar2_index",u"upper_bound",u"var_index"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"lower_bound",b"lower_bound",u"upper_bound",b"upper_bound"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"coefficient",b"coefficient",u"lower_bound",b"lower_bound",u"qcoefficient",b"qcoefficient",u"qvar1_index",b"qvar1_index",u"qvar2_index",b"qvar2_index",u"upper_bound",b"upper_bound",u"var_index",b"var_index"]) -> None: ...

class MPAbsConstraint(google___protobuf___message___Message):
    var_index = ... # type: int
    resultant_var_index = ... # type: int

    def __init__(self,
        *,
        var_index : typing___Optional[int] = None,
        resultant_var_index : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPAbsConstraint: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"resultant_var_index",u"var_index"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"resultant_var_index",u"var_index"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"resultant_var_index",b"resultant_var_index",u"var_index",b"var_index"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"resultant_var_index",b"resultant_var_index",u"var_index",b"var_index"]) -> None: ...

class MPArrayConstraint(google___protobuf___message___Message):
    var_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    resultant_var_index = ... # type: int

    def __init__(self,
        *,
        var_index : typing___Optional[typing___Iterable[int]] = None,
        resultant_var_index : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPArrayConstraint: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"resultant_var_index"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"resultant_var_index",u"var_index"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"resultant_var_index",b"resultant_var_index"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"resultant_var_index",b"resultant_var_index",u"var_index",b"var_index"]) -> None: ...

class MPArrayWithConstantConstraint(google___protobuf___message___Message):
    var_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    constant = ... # type: float
    resultant_var_index = ... # type: int

    def __init__(self,
        *,
        var_index : typing___Optional[typing___Iterable[int]] = None,
        constant : typing___Optional[float] = None,
        resultant_var_index : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPArrayWithConstantConstraint: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"constant",u"resultant_var_index"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"constant",u"resultant_var_index",u"var_index"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"constant",b"constant",u"resultant_var_index",b"resultant_var_index"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"constant",b"constant",u"resultant_var_index",b"resultant_var_index",u"var_index",b"var_index"]) -> None: ...

class MPQuadraticObjective(google___protobuf___message___Message):
    qvar1_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    qvar2_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    coefficient = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[float]

    def __init__(self,
        *,
        qvar1_index : typing___Optional[typing___Iterable[int]] = None,
        qvar2_index : typing___Optional[typing___Iterable[int]] = None,
        coefficient : typing___Optional[typing___Iterable[float]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPQuadraticObjective: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"coefficient",u"qvar1_index",u"qvar2_index"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"coefficient",b"coefficient",u"qvar1_index",b"qvar1_index",u"qvar2_index",b"qvar2_index"]) -> None: ...

class PartialVariableAssignment(google___protobuf___message___Message):
    var_index = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    var_value = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[float]

    def __init__(self,
        *,
        var_index : typing___Optional[typing___Iterable[int]] = None,
        var_value : typing___Optional[typing___Iterable[float]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> PartialVariableAssignment: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"var_index",u"var_value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"var_index",b"var_index",u"var_value",b"var_value"]) -> None: ...

class MPModelProto(google___protobuf___message___Message):
    maximize = ... # type: bool
    objective_offset = ... # type: float
    name = ... # type: typing___Text

    @property
    def variable(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[MPVariableProto]: ...

    @property
    def constraint(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[MPConstraintProto]: ...

    @property
    def general_constraint(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[MPGeneralConstraintProto]: ...

    @property
    def quadratic_objective(self) -> MPQuadraticObjective: ...

    @property
    def solution_hint(self) -> PartialVariableAssignment: ...

    def __init__(self,
        *,
        variable : typing___Optional[typing___Iterable[MPVariableProto]] = None,
        constraint : typing___Optional[typing___Iterable[MPConstraintProto]] = None,
        general_constraint : typing___Optional[typing___Iterable[MPGeneralConstraintProto]] = None,
        maximize : typing___Optional[bool] = None,
        objective_offset : typing___Optional[float] = None,
        quadratic_objective : typing___Optional[MPQuadraticObjective] = None,
        name : typing___Optional[typing___Text] = None,
        solution_hint : typing___Optional[PartialVariableAssignment] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPModelProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"maximize",u"name",u"objective_offset",u"quadratic_objective",u"solution_hint"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"constraint",u"general_constraint",u"maximize",u"name",u"objective_offset",u"quadratic_objective",u"solution_hint",u"variable"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"maximize",b"maximize",u"name",b"name",u"objective_offset",b"objective_offset",u"quadratic_objective",b"quadratic_objective",u"solution_hint",b"solution_hint"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"constraint",b"constraint",u"general_constraint",b"general_constraint",u"maximize",b"maximize",u"name",b"name",u"objective_offset",b"objective_offset",u"quadratic_objective",b"quadratic_objective",u"solution_hint",b"solution_hint",u"variable",b"variable"]) -> None: ...

class OptionalDouble(google___protobuf___message___Message):
    value = ... # type: float

    def __init__(self,
        *,
        value : typing___Optional[float] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> OptionalDouble: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class MPSolverCommonParameters(google___protobuf___message___Message):
    class LPAlgorithmValues(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> MPSolverCommonParameters.LPAlgorithmValues: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[MPSolverCommonParameters.LPAlgorithmValues]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, MPSolverCommonParameters.LPAlgorithmValues]]: ...
        LP_ALGO_UNSPECIFIED = typing___cast(MPSolverCommonParameters.LPAlgorithmValues, 0)
        LP_ALGO_DUAL = typing___cast(MPSolverCommonParameters.LPAlgorithmValues, 1)
        LP_ALGO_PRIMAL = typing___cast(MPSolverCommonParameters.LPAlgorithmValues, 2)
        LP_ALGO_BARRIER = typing___cast(MPSolverCommonParameters.LPAlgorithmValues, 3)
    LP_ALGO_UNSPECIFIED = typing___cast(MPSolverCommonParameters.LPAlgorithmValues, 0)
    LP_ALGO_DUAL = typing___cast(MPSolverCommonParameters.LPAlgorithmValues, 1)
    LP_ALGO_PRIMAL = typing___cast(MPSolverCommonParameters.LPAlgorithmValues, 2)
    LP_ALGO_BARRIER = typing___cast(MPSolverCommonParameters.LPAlgorithmValues, 3)

    lp_algorithm = ... # type: MPSolverCommonParameters.LPAlgorithmValues
    presolve = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean
    scaling = ... # type: ortools___util___optional_boolean_pb2___OptionalBoolean

    @property
    def relative_mip_gap(self) -> OptionalDouble: ...

    @property
    def primal_tolerance(self) -> OptionalDouble: ...

    @property
    def dual_tolerance(self) -> OptionalDouble: ...

    def __init__(self,
        *,
        relative_mip_gap : typing___Optional[OptionalDouble] = None,
        primal_tolerance : typing___Optional[OptionalDouble] = None,
        dual_tolerance : typing___Optional[OptionalDouble] = None,
        lp_algorithm : typing___Optional[MPSolverCommonParameters.LPAlgorithmValues] = None,
        presolve : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
        scaling : typing___Optional[ortools___util___optional_boolean_pb2___OptionalBoolean] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPSolverCommonParameters: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"dual_tolerance",u"lp_algorithm",u"presolve",u"primal_tolerance",u"relative_mip_gap",u"scaling"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"dual_tolerance",u"lp_algorithm",u"presolve",u"primal_tolerance",u"relative_mip_gap",u"scaling"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"dual_tolerance",b"dual_tolerance",u"lp_algorithm",b"lp_algorithm",u"presolve",b"presolve",u"primal_tolerance",b"primal_tolerance",u"relative_mip_gap",b"relative_mip_gap",u"scaling",b"scaling"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"dual_tolerance",b"dual_tolerance",u"lp_algorithm",b"lp_algorithm",u"presolve",b"presolve",u"primal_tolerance",b"primal_tolerance",u"relative_mip_gap",b"relative_mip_gap",u"scaling",b"scaling"]) -> None: ...

class MPModelDeltaProto(google___protobuf___message___Message):
    class VariableOverridesEntry(google___protobuf___message___Message):
        key = ... # type: int

        @property
        def value(self) -> MPVariableProto: ...

        def __init__(self,
            *,
            key : typing___Optional[int] = None,
            value : typing___Optional[MPVariableProto] = None,
            ) -> None: ...
        @classmethod
        def FromString(cls, s: bytes) -> MPModelDeltaProto.VariableOverridesEntry: ...
        def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        if sys.version_info >= (3,):
            def HasField(self, field_name: typing_extensions___Literal[u"key",u"value"]) -> bool: ...
            def ClearField(self, field_name: typing_extensions___Literal[u"key",u"value"]) -> None: ...
        else:
            def HasField(self, field_name: typing_extensions___Literal[u"key",b"key",u"value",b"value"]) -> bool: ...
            def ClearField(self, field_name: typing_extensions___Literal[u"key",b"key",u"value",b"value"]) -> None: ...

    class ConstraintOverridesEntry(google___protobuf___message___Message):
        key = ... # type: int

        @property
        def value(self) -> MPConstraintProto: ...

        def __init__(self,
            *,
            key : typing___Optional[int] = None,
            value : typing___Optional[MPConstraintProto] = None,
            ) -> None: ...
        @classmethod
        def FromString(cls, s: bytes) -> MPModelDeltaProto.ConstraintOverridesEntry: ...
        def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        if sys.version_info >= (3,):
            def HasField(self, field_name: typing_extensions___Literal[u"key",u"value"]) -> bool: ...
            def ClearField(self, field_name: typing_extensions___Literal[u"key",u"value"]) -> None: ...
        else:
            def HasField(self, field_name: typing_extensions___Literal[u"key",b"key",u"value",b"value"]) -> bool: ...
            def ClearField(self, field_name: typing_extensions___Literal[u"key",b"key",u"value",b"value"]) -> None: ...

    baseline_model_file_path = ... # type: typing___Text

    @property
    def variable_overrides(self) -> typing___MutableMapping[int, MPVariableProto]: ...

    @property
    def constraint_overrides(self) -> typing___MutableMapping[int, MPConstraintProto]: ...

    def __init__(self,
        *,
        baseline_model_file_path : typing___Optional[typing___Text] = None,
        variable_overrides : typing___Optional[typing___Mapping[int, MPVariableProto]] = None,
        constraint_overrides : typing___Optional[typing___Mapping[int, MPConstraintProto]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPModelDeltaProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"baseline_model_file_path"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"baseline_model_file_path",u"constraint_overrides",u"variable_overrides"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"baseline_model_file_path",b"baseline_model_file_path"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"baseline_model_file_path",b"baseline_model_file_path",u"constraint_overrides",b"constraint_overrides",u"variable_overrides",b"variable_overrides"]) -> None: ...

class MPModelRequest(google___protobuf___message___Message):
    class SolverType(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> MPModelRequest.SolverType: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[MPModelRequest.SolverType]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, MPModelRequest.SolverType]]: ...
        GLOP_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 2)
        CLP_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 0)
        GLPK_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 1)
        GUROBI_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 6)
        XPRESS_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 101)
        CPLEX_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 10)
        SCIP_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 3)
        GLPK_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 4)
        CBC_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 5)
        GUROBI_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 7)
        XPRESS_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 102)
        CPLEX_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 11)
        BOP_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 12)
        SAT_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 14)
        KNAPSACK_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 13)
    GLOP_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 2)
    CLP_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 0)
    GLPK_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 1)
    GUROBI_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 6)
    XPRESS_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 101)
    CPLEX_LINEAR_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 10)
    SCIP_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 3)
    GLPK_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 4)
    CBC_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 5)
    GUROBI_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 7)
    XPRESS_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 102)
    CPLEX_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 11)
    BOP_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 12)
    SAT_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 14)
    KNAPSACK_MIXED_INTEGER_PROGRAMMING = typing___cast(MPModelRequest.SolverType, 13)

    solver_type = ... # type: MPModelRequest.SolverType
    solver_time_limit_seconds = ... # type: float
    enable_internal_solver_output = ... # type: bool
    solver_specific_parameters = ... # type: typing___Text

    @property
    def model(self) -> MPModelProto: ...

    @property
    def model_delta(self) -> MPModelDeltaProto: ...

    def __init__(self,
        *,
        model : typing___Optional[MPModelProto] = None,
        solver_type : typing___Optional[MPModelRequest.SolverType] = None,
        solver_time_limit_seconds : typing___Optional[float] = None,
        enable_internal_solver_output : typing___Optional[bool] = None,
        solver_specific_parameters : typing___Optional[typing___Text] = None,
        model_delta : typing___Optional[MPModelDeltaProto] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPModelRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"enable_internal_solver_output",u"model",u"model_delta",u"solver_specific_parameters",u"solver_time_limit_seconds",u"solver_type"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"enable_internal_solver_output",u"model",u"model_delta",u"solver_specific_parameters",u"solver_time_limit_seconds",u"solver_type"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"enable_internal_solver_output",b"enable_internal_solver_output",u"model",b"model",u"model_delta",b"model_delta",u"solver_specific_parameters",b"solver_specific_parameters",u"solver_time_limit_seconds",b"solver_time_limit_seconds",u"solver_type",b"solver_type"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"enable_internal_solver_output",b"enable_internal_solver_output",u"model",b"model",u"model_delta",b"model_delta",u"solver_specific_parameters",b"solver_specific_parameters",u"solver_time_limit_seconds",b"solver_time_limit_seconds",u"solver_type",b"solver_type"]) -> None: ...

class MPSolutionResponse(google___protobuf___message___Message):
    status = ... # type: MPSolverResponseStatus
    status_str = ... # type: typing___Text
    objective_value = ... # type: float
    best_objective_bound = ... # type: float
    variable_value = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[float]
    dual_value = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[float]
    reduced_cost = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[float]

    def __init__(self,
        *,
        status : typing___Optional[MPSolverResponseStatus] = None,
        status_str : typing___Optional[typing___Text] = None,
        objective_value : typing___Optional[float] = None,
        best_objective_bound : typing___Optional[float] = None,
        variable_value : typing___Optional[typing___Iterable[float]] = None,
        dual_value : typing___Optional[typing___Iterable[float]] = None,
        reduced_cost : typing___Optional[typing___Iterable[float]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> MPSolutionResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"best_objective_bound",u"objective_value",u"status",u"status_str"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"best_objective_bound",u"dual_value",u"objective_value",u"reduced_cost",u"status",u"status_str",u"variable_value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"best_objective_bound",b"best_objective_bound",u"objective_value",b"objective_value",u"status",b"status",u"status_str",b"status_str"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"best_objective_bound",b"best_objective_bound",u"dual_value",b"dual_value",u"objective_value",b"objective_value",u"reduced_cost",b"reduced_cost",u"status",b"status",u"status_str",b"status_str",u"variable_value",b"variable_value"]) -> None: ...



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    EnumDescriptor as google___protobuf___descriptor___EnumDescriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    List as typing___List,
    Optional as typing___Optional,
    Text as typing___Text,
    Tuple as typing___Tuple,
    cast as typing___cast,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


class CpSolverStatus(int):
    DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
    @classmethod
    def Name(cls, number: int) -> str: ...
    @classmethod
    def Value(cls, name: str) -> CpSolverStatus: ...
    @classmethod
    def keys(cls) -> typing___List[str]: ...
    @classmethod
    def values(cls) -> typing___List[CpSolverStatus]: ...
    @classmethod
    def items(cls) -> typing___List[typing___Tuple[str, CpSolverStatus]]: ...
    UNKNOWN = typing___cast(CpSolverStatus, 0)
    MODEL_INVALID = typing___cast(CpSolverStatus, 1)
    FEASIBLE = typing___cast(CpSolverStatus, 2)
    INFEASIBLE = typing___cast(CpSolverStatus, 3)
    OPTIMAL = typing___cast(CpSolverStatus, 4)
UNKNOWN = typing___cast(CpSolverStatus, 0)
MODEL_INVALID = typing___cast(CpSolverStatus, 1)
FEASIBLE = typing___cast(CpSolverStatus, 2)
INFEASIBLE = typing___cast(CpSolverStatus, 3)
OPTIMAL = typing___cast(CpSolverStatus, 4)

class IntegerVariableProto(google___protobuf___message___Message):
    name = ... # type: typing___Text
    domain = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        domain : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> IntegerVariableProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"domain",u"name"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"domain",b"domain",u"name",b"name"]) -> None: ...

class BoolArgumentProto(google___protobuf___message___Message):
    literals = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        literals : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> BoolArgumentProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"literals"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"literals",b"literals"]) -> None: ...

class IntegerArgumentProto(google___protobuf___message___Message):
    target = ... # type: int
    vars = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        target : typing___Optional[int] = None,
        vars : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> IntegerArgumentProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"target",u"vars"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"target",b"target",u"vars",b"vars"]) -> None: ...

class LinearExpressionProto(google___protobuf___message___Message):
    vars = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    coeffs = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    offset = ... # type: int

    def __init__(self,
        *,
        vars : typing___Optional[typing___Iterable[int]] = None,
        coeffs : typing___Optional[typing___Iterable[int]] = None,
        offset : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> LinearExpressionProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"coeffs",u"offset",u"vars"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"coeffs",b"coeffs",u"offset",b"offset",u"vars",b"vars"]) -> None: ...

class LinearArgumentProto(google___protobuf___message___Message):

    @property
    def target(self) -> LinearExpressionProto: ...

    @property
    def exprs(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[LinearExpressionProto]: ...

    def __init__(self,
        *,
        target : typing___Optional[LinearExpressionProto] = None,
        exprs : typing___Optional[typing___Iterable[LinearExpressionProto]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> LinearArgumentProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"target"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"exprs",u"target"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"target",b"target"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"exprs",b"exprs",u"target",b"target"]) -> None: ...

class AllDifferentConstraintProto(google___protobuf___message___Message):
    vars = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        vars : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> AllDifferentConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"vars"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"vars",b"vars"]) -> None: ...

class LinearConstraintProto(google___protobuf___message___Message):
    vars = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    coeffs = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    domain = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        vars : typing___Optional[typing___Iterable[int]] = None,
        coeffs : typing___Optional[typing___Iterable[int]] = None,
        domain : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> LinearConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"coeffs",u"domain",u"vars"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"coeffs",b"coeffs",u"domain",b"domain",u"vars",b"vars"]) -> None: ...

class ElementConstraintProto(google___protobuf___message___Message):
    index = ... # type: int
    target = ... # type: int
    vars = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        index : typing___Optional[int] = None,
        target : typing___Optional[int] = None,
        vars : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> ElementConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"index",u"target",u"vars"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"index",b"index",u"target",b"target",u"vars",b"vars"]) -> None: ...

class IntervalConstraintProto(google___protobuf___message___Message):
    start = ... # type: int
    end = ... # type: int
    size = ... # type: int

    def __init__(self,
        *,
        start : typing___Optional[int] = None,
        end : typing___Optional[int] = None,
        size : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> IntervalConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"end",u"size",u"start"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"end",b"end",u"size",b"size",u"start",b"start"]) -> None: ...

class NoOverlapConstraintProto(google___protobuf___message___Message):
    intervals = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        intervals : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> NoOverlapConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"intervals"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"intervals",b"intervals"]) -> None: ...

class NoOverlap2DConstraintProto(google___protobuf___message___Message):
    x_intervals = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    y_intervals = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    boxes_with_null_area_can_overlap = ... # type: bool

    def __init__(self,
        *,
        x_intervals : typing___Optional[typing___Iterable[int]] = None,
        y_intervals : typing___Optional[typing___Iterable[int]] = None,
        boxes_with_null_area_can_overlap : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> NoOverlap2DConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"boxes_with_null_area_can_overlap",u"x_intervals",u"y_intervals"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"boxes_with_null_area_can_overlap",b"boxes_with_null_area_can_overlap",u"x_intervals",b"x_intervals",u"y_intervals",b"y_intervals"]) -> None: ...

class CumulativeConstraintProto(google___protobuf___message___Message):
    capacity = ... # type: int
    intervals = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    demands = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        capacity : typing___Optional[int] = None,
        intervals : typing___Optional[typing___Iterable[int]] = None,
        demands : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> CumulativeConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"capacity",u"demands",u"intervals"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"capacity",b"capacity",u"demands",b"demands",u"intervals",b"intervals"]) -> None: ...

class ReservoirConstraintProto(google___protobuf___message___Message):
    min_level = ... # type: int
    max_level = ... # type: int
    times = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    demands = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    actives = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        min_level : typing___Optional[int] = None,
        max_level : typing___Optional[int] = None,
        times : typing___Optional[typing___Iterable[int]] = None,
        demands : typing___Optional[typing___Iterable[int]] = None,
        actives : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> ReservoirConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"actives",u"demands",u"max_level",u"min_level",u"times"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"actives",b"actives",u"demands",b"demands",u"max_level",b"max_level",u"min_level",b"min_level",u"times",b"times"]) -> None: ...

class CircuitConstraintProto(google___protobuf___message___Message):
    tails = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    heads = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    literals = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        tails : typing___Optional[typing___Iterable[int]] = None,
        heads : typing___Optional[typing___Iterable[int]] = None,
        literals : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> CircuitConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"heads",u"literals",u"tails"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"heads",b"heads",u"literals",b"literals",u"tails",b"tails"]) -> None: ...

class RoutesConstraintProto(google___protobuf___message___Message):
    tails = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    heads = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    literals = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    demands = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    capacity = ... # type: int

    def __init__(self,
        *,
        tails : typing___Optional[typing___Iterable[int]] = None,
        heads : typing___Optional[typing___Iterable[int]] = None,
        literals : typing___Optional[typing___Iterable[int]] = None,
        demands : typing___Optional[typing___Iterable[int]] = None,
        capacity : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> RoutesConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"capacity",u"demands",u"heads",u"literals",u"tails"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"capacity",b"capacity",u"demands",b"demands",u"heads",b"heads",u"literals",b"literals",u"tails",b"tails"]) -> None: ...

class CircuitCoveringConstraintProto(google___protobuf___message___Message):
    nexts = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    distinguished_nodes = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        nexts : typing___Optional[typing___Iterable[int]] = None,
        distinguished_nodes : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> CircuitCoveringConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"distinguished_nodes",u"nexts"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"distinguished_nodes",b"distinguished_nodes",u"nexts",b"nexts"]) -> None: ...

class TableConstraintProto(google___protobuf___message___Message):
    vars = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    values = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    negated = ... # type: bool

    def __init__(self,
        *,
        vars : typing___Optional[typing___Iterable[int]] = None,
        values : typing___Optional[typing___Iterable[int]] = None,
        negated : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> TableConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"negated",u"values",u"vars"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"negated",b"negated",u"values",b"values",u"vars",b"vars"]) -> None: ...

class InverseConstraintProto(google___protobuf___message___Message):
    f_direct = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    f_inverse = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        f_direct : typing___Optional[typing___Iterable[int]] = None,
        f_inverse : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> InverseConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"f_direct",u"f_inverse"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"f_direct",b"f_direct",u"f_inverse",b"f_inverse"]) -> None: ...

class AutomatonConstraintProto(google___protobuf___message___Message):
    starting_state = ... # type: int
    final_states = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    transition_tail = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    transition_head = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    transition_label = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    vars = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        starting_state : typing___Optional[int] = None,
        final_states : typing___Optional[typing___Iterable[int]] = None,
        transition_tail : typing___Optional[typing___Iterable[int]] = None,
        transition_head : typing___Optional[typing___Iterable[int]] = None,
        transition_label : typing___Optional[typing___Iterable[int]] = None,
        vars : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> AutomatonConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"final_states",u"starting_state",u"transition_head",u"transition_label",u"transition_tail",u"vars"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"final_states",b"final_states",u"starting_state",b"starting_state",u"transition_head",b"transition_head",u"transition_label",b"transition_label",u"transition_tail",b"transition_tail",u"vars",b"vars"]) -> None: ...

class ConstraintProto(google___protobuf___message___Message):
    name = ... # type: typing___Text
    enforcement_literal = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    @property
    def bool_or(self) -> BoolArgumentProto: ...

    @property
    def bool_and(self) -> BoolArgumentProto: ...

    @property
    def at_most_one(self) -> BoolArgumentProto: ...

    @property
    def bool_xor(self) -> BoolArgumentProto: ...

    @property
    def int_div(self) -> IntegerArgumentProto: ...

    @property
    def int_mod(self) -> IntegerArgumentProto: ...

    @property
    def int_max(self) -> IntegerArgumentProto: ...

    @property
    def lin_max(self) -> LinearArgumentProto: ...

    @property
    def int_min(self) -> IntegerArgumentProto: ...

    @property
    def lin_min(self) -> LinearArgumentProto: ...

    @property
    def int_prod(self) -> IntegerArgumentProto: ...

    @property
    def linear(self) -> LinearConstraintProto: ...

    @property
    def all_diff(self) -> AllDifferentConstraintProto: ...

    @property
    def element(self) -> ElementConstraintProto: ...

    @property
    def circuit(self) -> CircuitConstraintProto: ...

    @property
    def routes(self) -> RoutesConstraintProto: ...

    @property
    def circuit_covering(self) -> CircuitCoveringConstraintProto: ...

    @property
    def table(self) -> TableConstraintProto: ...

    @property
    def automaton(self) -> AutomatonConstraintProto: ...

    @property
    def inverse(self) -> InverseConstraintProto: ...

    @property
    def reservoir(self) -> ReservoirConstraintProto: ...

    @property
    def interval(self) -> IntervalConstraintProto: ...

    @property
    def no_overlap(self) -> NoOverlapConstraintProto: ...

    @property
    def no_overlap_2d(self) -> NoOverlap2DConstraintProto: ...

    @property
    def cumulative(self) -> CumulativeConstraintProto: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        enforcement_literal : typing___Optional[typing___Iterable[int]] = None,
        bool_or : typing___Optional[BoolArgumentProto] = None,
        bool_and : typing___Optional[BoolArgumentProto] = None,
        at_most_one : typing___Optional[BoolArgumentProto] = None,
        bool_xor : typing___Optional[BoolArgumentProto] = None,
        int_div : typing___Optional[IntegerArgumentProto] = None,
        int_mod : typing___Optional[IntegerArgumentProto] = None,
        int_max : typing___Optional[IntegerArgumentProto] = None,
        lin_max : typing___Optional[LinearArgumentProto] = None,
        int_min : typing___Optional[IntegerArgumentProto] = None,
        lin_min : typing___Optional[LinearArgumentProto] = None,
        int_prod : typing___Optional[IntegerArgumentProto] = None,
        linear : typing___Optional[LinearConstraintProto] = None,
        all_diff : typing___Optional[AllDifferentConstraintProto] = None,
        element : typing___Optional[ElementConstraintProto] = None,
        circuit : typing___Optional[CircuitConstraintProto] = None,
        routes : typing___Optional[RoutesConstraintProto] = None,
        circuit_covering : typing___Optional[CircuitCoveringConstraintProto] = None,
        table : typing___Optional[TableConstraintProto] = None,
        automaton : typing___Optional[AutomatonConstraintProto] = None,
        inverse : typing___Optional[InverseConstraintProto] = None,
        reservoir : typing___Optional[ReservoirConstraintProto] = None,
        interval : typing___Optional[IntervalConstraintProto] = None,
        no_overlap : typing___Optional[NoOverlapConstraintProto] = None,
        no_overlap_2d : typing___Optional[NoOverlap2DConstraintProto] = None,
        cumulative : typing___Optional[CumulativeConstraintProto] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> ConstraintProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"all_diff",u"at_most_one",u"automaton",u"bool_and",u"bool_or",u"bool_xor",u"circuit",u"circuit_covering",u"constraint",u"cumulative",u"element",u"int_div",u"int_max",u"int_min",u"int_mod",u"int_prod",u"interval",u"inverse",u"lin_max",u"lin_min",u"linear",u"no_overlap",u"no_overlap_2d",u"reservoir",u"routes",u"table"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"all_diff",u"at_most_one",u"automaton",u"bool_and",u"bool_or",u"bool_xor",u"circuit",u"circuit_covering",u"constraint",u"cumulative",u"element",u"enforcement_literal",u"int_div",u"int_max",u"int_min",u"int_mod",u"int_prod",u"interval",u"inverse",u"lin_max",u"lin_min",u"linear",u"name",u"no_overlap",u"no_overlap_2d",u"reservoir",u"routes",u"table"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"all_diff",b"all_diff",u"at_most_one",b"at_most_one",u"automaton",b"automaton",u"bool_and",b"bool_and",u"bool_or",b"bool_or",u"bool_xor",b"bool_xor",u"circuit",b"circuit",u"circuit_covering",b"circuit_covering",u"constraint",b"constraint",u"cumulative",b"cumulative",u"element",b"element",u"int_div",b"int_div",u"int_max",b"int_max",u"int_min",b"int_min",u"int_mod",b"int_mod",u"int_prod",b"int_prod",u"interval",b"interval",u"inverse",b"inverse",u"lin_max",b"lin_max",u"lin_min",b"lin_min",u"linear",b"linear",u"no_overlap",b"no_overlap",u"no_overlap_2d",b"no_overlap_2d",u"reservoir",b"reservoir",u"routes",b"routes",u"table",b"table"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"all_diff",b"all_diff",u"at_most_one",b"at_most_one",u"automaton",b"automaton",u"bool_and",b"bool_and",u"bool_or",b"bool_or",u"bool_xor",b"bool_xor",u"circuit",b"circuit",u"circuit_covering",b"circuit_covering",u"constraint",b"constraint",u"cumulative",b"cumulative",u"element",b"element",u"enforcement_literal",b"enforcement_literal",u"int_div",b"int_div",u"int_max",b"int_max",u"int_min",b"int_min",u"int_mod",b"int_mod",u"int_prod",b"int_prod",u"interval",b"interval",u"inverse",b"inverse",u"lin_max",b"lin_max",u"lin_min",b"lin_min",u"linear",b"linear",u"name",b"name",u"no_overlap",b"no_overlap",u"no_overlap_2d",b"no_overlap_2d",u"reservoir",b"reservoir",u"routes",b"routes",u"table",b"table"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions___Literal[u"constraint",b"constraint"]) -> typing_extensions___Literal["bool_or","bool_and","at_most_one","bool_xor","int_div","int_mod","int_max","lin_max","int_min","lin_min","int_prod","linear","all_diff","element","circuit","routes","circuit_covering","table","automaton","inverse","reservoir","interval","no_overlap","no_overlap_2d","cumulative"]: ...

class CpObjectiveProto(google___protobuf___message___Message):
    vars = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    coeffs = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    offset = ... # type: float
    scaling_factor = ... # type: float
    domain = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        vars : typing___Optional[typing___Iterable[int]] = None,
        coeffs : typing___Optional[typing___Iterable[int]] = None,
        offset : typing___Optional[float] = None,
        scaling_factor : typing___Optional[float] = None,
        domain : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> CpObjectiveProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"coeffs",u"domain",u"offset",u"scaling_factor",u"vars"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"coeffs",b"coeffs",u"domain",b"domain",u"offset",b"offset",u"scaling_factor",b"scaling_factor",u"vars",b"vars"]) -> None: ...

class DecisionStrategyProto(google___protobuf___message___Message):
    class VariableSelectionStrategy(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> DecisionStrategyProto.VariableSelectionStrategy: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[DecisionStrategyProto.VariableSelectionStrategy]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, DecisionStrategyProto.VariableSelectionStrategy]]: ...
        CHOOSE_FIRST = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 0)
        CHOOSE_LOWEST_MIN = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 1)
        CHOOSE_HIGHEST_MAX = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 2)
        CHOOSE_MIN_DOMAIN_SIZE = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 3)
        CHOOSE_MAX_DOMAIN_SIZE = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 4)
    CHOOSE_FIRST = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 0)
    CHOOSE_LOWEST_MIN = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 1)
    CHOOSE_HIGHEST_MAX = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 2)
    CHOOSE_MIN_DOMAIN_SIZE = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 3)
    CHOOSE_MAX_DOMAIN_SIZE = typing___cast(DecisionStrategyProto.VariableSelectionStrategy, 4)

    class DomainReductionStrategy(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> DecisionStrategyProto.DomainReductionStrategy: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[DecisionStrategyProto.DomainReductionStrategy]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, DecisionStrategyProto.DomainReductionStrategy]]: ...
        SELECT_MIN_VALUE = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 0)
        SELECT_MAX_VALUE = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 1)
        SELECT_LOWER_HALF = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 2)
        SELECT_UPPER_HALF = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 3)
        SELECT_MEDIAN_VALUE = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 4)
    SELECT_MIN_VALUE = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 0)
    SELECT_MAX_VALUE = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 1)
    SELECT_LOWER_HALF = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 2)
    SELECT_UPPER_HALF = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 3)
    SELECT_MEDIAN_VALUE = typing___cast(DecisionStrategyProto.DomainReductionStrategy, 4)

    class AffineTransformation(google___protobuf___message___Message):
        var = ... # type: int
        offset = ... # type: int
        positive_coeff = ... # type: int

        def __init__(self,
            *,
            var : typing___Optional[int] = None,
            offset : typing___Optional[int] = None,
            positive_coeff : typing___Optional[int] = None,
            ) -> None: ...
        @classmethod
        def FromString(cls, s: bytes) -> DecisionStrategyProto.AffineTransformation: ...
        def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
        if sys.version_info >= (3,):
            def ClearField(self, field_name: typing_extensions___Literal[u"offset",u"positive_coeff",u"var"]) -> None: ...
        else:
            def ClearField(self, field_name: typing_extensions___Literal[u"offset",b"offset",u"positive_coeff",b"positive_coeff",u"var",b"var"]) -> None: ...

    variables = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    variable_selection_strategy = ... # type: DecisionStrategyProto.VariableSelectionStrategy
    domain_reduction_strategy = ... # type: DecisionStrategyProto.DomainReductionStrategy

    @property
    def transformations(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[DecisionStrategyProto.AffineTransformation]: ...

    def __init__(self,
        *,
        variables : typing___Optional[typing___Iterable[int]] = None,
        variable_selection_strategy : typing___Optional[DecisionStrategyProto.VariableSelectionStrategy] = None,
        domain_reduction_strategy : typing___Optional[DecisionStrategyProto.DomainReductionStrategy] = None,
        transformations : typing___Optional[typing___Iterable[DecisionStrategyProto.AffineTransformation]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DecisionStrategyProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"domain_reduction_strategy",u"transformations",u"variable_selection_strategy",u"variables"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"domain_reduction_strategy",b"domain_reduction_strategy",u"transformations",b"transformations",u"variable_selection_strategy",b"variable_selection_strategy",u"variables",b"variables"]) -> None: ...

class PartialVariableAssignment(google___protobuf___message___Message):
    vars = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    values = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        *,
        vars : typing___Optional[typing___Iterable[int]] = None,
        values : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> PartialVariableAssignment: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"values",u"vars"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"values",b"values",u"vars",b"vars"]) -> None: ...

class CpModelProto(google___protobuf___message___Message):
    name = ... # type: typing___Text
    assumptions = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    @property
    def variables(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[IntegerVariableProto]: ...

    @property
    def constraints(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[ConstraintProto]: ...

    @property
    def objective(self) -> CpObjectiveProto: ...

    @property
    def search_strategy(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[DecisionStrategyProto]: ...

    @property
    def solution_hint(self) -> PartialVariableAssignment: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        variables : typing___Optional[typing___Iterable[IntegerVariableProto]] = None,
        constraints : typing___Optional[typing___Iterable[ConstraintProto]] = None,
        objective : typing___Optional[CpObjectiveProto] = None,
        search_strategy : typing___Optional[typing___Iterable[DecisionStrategyProto]] = None,
        solution_hint : typing___Optional[PartialVariableAssignment] = None,
        assumptions : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> CpModelProto: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"objective",u"solution_hint"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"assumptions",u"constraints",u"name",u"objective",u"search_strategy",u"solution_hint",u"variables"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"objective",b"objective",u"solution_hint",b"solution_hint"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"assumptions",b"assumptions",u"constraints",b"constraints",u"name",b"name",u"objective",b"objective",u"search_strategy",b"search_strategy",u"solution_hint",b"solution_hint",u"variables",b"variables"]) -> None: ...

class CpSolverResponse(google___protobuf___message___Message):
    status = ... # type: CpSolverStatus
    solution = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    objective_value = ... # type: float
    best_objective_bound = ... # type: float
    solution_lower_bounds = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    solution_upper_bounds = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    sufficient_assumptions_for_infeasibility = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]
    all_solutions_were_found = ... # type: bool
    num_booleans = ... # type: int
    num_conflicts = ... # type: int
    num_branches = ... # type: int
    num_binary_propagations = ... # type: int
    num_integer_propagations = ... # type: int
    wall_time = ... # type: float
    user_time = ... # type: float
    deterministic_time = ... # type: float
    primal_integral = ... # type: float
    solution_info = ... # type: typing___Text

    @property
    def tightened_variables(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[IntegerVariableProto]: ...

    def __init__(self,
        *,
        status : typing___Optional[CpSolverStatus] = None,
        solution : typing___Optional[typing___Iterable[int]] = None,
        objective_value : typing___Optional[float] = None,
        best_objective_bound : typing___Optional[float] = None,
        solution_lower_bounds : typing___Optional[typing___Iterable[int]] = None,
        solution_upper_bounds : typing___Optional[typing___Iterable[int]] = None,
        tightened_variables : typing___Optional[typing___Iterable[IntegerVariableProto]] = None,
        sufficient_assumptions_for_infeasibility : typing___Optional[typing___Iterable[int]] = None,
        all_solutions_were_found : typing___Optional[bool] = None,
        num_booleans : typing___Optional[int] = None,
        num_conflicts : typing___Optional[int] = None,
        num_branches : typing___Optional[int] = None,
        num_binary_propagations : typing___Optional[int] = None,
        num_integer_propagations : typing___Optional[int] = None,
        wall_time : typing___Optional[float] = None,
        user_time : typing___Optional[float] = None,
        deterministic_time : typing___Optional[float] = None,
        primal_integral : typing___Optional[float] = None,
        solution_info : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> CpSolverResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"all_solutions_were_found",u"best_objective_bound",u"deterministic_time",u"num_binary_propagations",u"num_booleans",u"num_branches",u"num_conflicts",u"num_integer_propagations",u"objective_value",u"primal_integral",u"solution",u"solution_info",u"solution_lower_bounds",u"solution_upper_bounds",u"status",u"sufficient_assumptions_for_infeasibility",u"tightened_variables",u"user_time",u"wall_time"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"all_solutions_were_found",b"all_solutions_were_found",u"best_objective_bound",b"best_objective_bound",u"deterministic_time",b"deterministic_time",u"num_binary_propagations",b"num_binary_propagations",u"num_booleans",b"num_booleans",u"num_branches",b"num_branches",u"num_conflicts",b"num_conflicts",u"num_integer_propagations",b"num_integer_propagations",u"objective_value",b"objective_value",u"primal_integral",b"primal_integral",u"solution",b"solution",u"solution_info",b"solution_info",u"solution_lower_bounds",b"solution_lower_bounds",u"solution_upper_bounds",b"solution_upper_bounds",u"status",b"status",u"sufficient_assumptions_for_infeasibility",b"sufficient_assumptions_for_infeasibility",u"tightened_variables",b"tightened_variables",u"user_time",b"user_time",u"wall_time",b"wall_time"]) -> None: ...
